AC_SUBST(MAKE)dnl
AC_SUBST(OBJC_RUNTIME)dnl
AC_SUBST(OBJC_RUNTIME_FLAG)dnl
AC_SUBST(BROKEN_COMPILER)dnl
AC_SUBST(BROKEN_BUILTIN_APPLY)dnl
AC_SUBST(FOUNDATION_LIBRARY)dnl
dnl
dnl
AC_DEFUN(AC_LANG_OBJECTIVE_C,
[AC_REQUIRE([AC_PROG_CC])dnl
define([AC_LANG], [AC_LANG_OBJECTIVE_C])dnl
ac_ext=m
# CFLAGS is not in ac_cpp because -g, -O, etc. are not valid cpp options.
ac_cpp='$CPP $OBJC_RUNTIME_FLAG'
ac_compile='${CC-cc} -c $OBJC_RUNTIME_FLAG $CFLAGS conftest.$ac_ext 1>&AC_FD_CC 2>&AC_FD_CC'
ac_link='${CC-cc} -o conftest $OBJC_RUNTIME_FLAG $CFLAGS $LDFLAGS conftest.$ac_ext $LIBS $OBJC_LIBS 1>&AC_FD_CC 2>&AC_FD_CC'
])dnl
dnl
dnl
AC_DEFUN(AC_CHECK_NESTED_FUNCTIONS,
[AC_REQUIRE([AC_PROG_CC])dnl
AC_MSG_CHECKING(whether nested functions work)
AC_CACHE_VAL(ac_cv_nested_functions,
[AC_LANG_SAVE[]dnl
AC_LANG_OBJECTIVE_C[]
AC_TRY_RUN([
f(void (*nested)())
{
    (*nested)();
}

main()
{
    int a = 0;
    void nested()
    {
	a = 1;
    }
    f(nested);
    if(a != 1)
	exit(1);
    exit(0);
}
], ac_cv_nested_functions=yes, ac_cv_nested_functions=no,
ac_cv_nested_functions=yes)
AC_LANG_RESTORE[]
if test $ac_cv_nested_functions = no; then
    AC_MSG_ERROR([
Exception handling facility cannot work without nested functions. You must
obtain the gcc version 2.6.3 or later and try again!
])
fi
])dnl
AC_MSG_RESULT(${ac_cv_nested_functions})
])dnl
dnl
dnl
AC_DEFUN(AC_BROKEN_BUILTIN_APPLY,
[AC_REQUIRE([AC_PROG_CC])dnl
AC_MSG_CHECKING(whether __builtin_apply and __builtin_return are broken)
AC_CACHE_VAL(ac_cv_broken_builtin_apply,
[AC_LANG_SAVE[]dnl
AC_LANG_OBJECTIVE_C[]
AC_TRY_RUN([
#include <objc/Object.h>

float value = 123.456;

@interface MyObject : Object
@end

@implementation MyObject
- (float)floatValue
{
    return value;
}
@end

@interface Forwarder : Object
{
    id object;
}
@end

@implementation Forwarder
- setObject:anObject
{
    object = anObject;
    return self;
}

- (void*)forward:(SEL)selector:(void*)argframe
{
    IMP imp = [object methodFor:@selector(floatValue)];
    void* retframe;
    void* frame = malloc(116);
    *(void**)frame = NULL;
    retframe = __builtin_apply((void(*)(void))imp, frame, 0);
    if(*(long double*)(((char*)retframe) + 8) == (long double)value)
	exit(0);
    exit(1);
}
@end

void main()
{
    id fwd = [[Forwarder alloc] setObject:[MyObject alloc]];
    [fwd floatValue];
    exit(0);
}
], ac_cv_broken_builtin_apply=no,
ac_cv_broken_builtin_apply=yes,
ac_cv_broken_builtin_apply=no)
AC_LANG_RESTORE[]
])dnl
AC_MSG_RESULT(${ac_cv_broken_builtin_apply})
BROKEN_BUILTIN_APPLY=${ac_cv_broken_builtin_apply}
if test $BROKEN_BUILTIN_APPLY = yes; then
    AC_DEFINE(BROKEN_BUILTIN_APPLY)
fi
])dnl
dnl
dnl
AC_DEFUN(AC_FIND_FOUNDATION,
[AC_REQUIRE([AC_PROG_CC])dnl
AC_REQUIRE([AC_C_CROSS])dnl
AC_MSG_CHECKING(for the Foundation library)
AC_CACHE_VAL(ac_cv_foundation_library,
[if test "$FOUNDATION" = ""; then
AC_LANG_SAVE[]dnl
AC_LANG_OBJECTIVE_C[]
AC_TRY_COMPILE(
#include <Foundation/preface.h>
,
, ac_cv_foundation_library="$ac_cv_foundation_library gnustep-base")
AC_TRY_COMPILE(
#include <Foundation/exceptions/FoundationException.h>
,
, ac_cv_foundation_library="$ac_cv_foundation_library libFoundation")
AC_TRY_COMPILE(
#import <Foundation/NSObject.h>
#import <synch.h>
,
, ac_cv_foundation_library=Sun)
AC_LANG_RESTORE[]dnl
AC_TRY_CPP(
#include <foundation/NSObject.h>
, ac_cv_foundation_library=foundation)
else
    ac_cv_foundation_library=$FOUNDATION
fi
])dnl
libs_found=`echo ${ac_cv_foundation_library} | awk '{print NF}' -`
if test $libs_found -gt 1; then
    echo "" 1>&2
    AC_MSG_ERROR([More than one Foundation library installed on your system. In the FOUNDATION variable you must specify exactly one of the following libraries: ${ac_cv_foundation_library}])
fi
FOUNDATION_LIBRARY=`echo ${ac_cv_foundation_library} | awk '{print $1}'`
case "$FOUNDATION_LIBRARY" in
    foundation)		OBJC_RUNTIME=next; LIBS="$LIBS -lFoundation_s"; AC_DEFINE(NeXT_foundation_LIBRARY);;
    Sun)		OBJC_RUNTIME=sun; LIBS="$LIBS -lFoundation"; AC_DEFINE(Sun_Foundation_LIBRARY);;
    gnustep-base)	LIBS="$LIBS -lgnustep-base"; AC_DEFINE(GNUSTEP_BASE_LIBRARY);;
    libFoundation)	LIBS="$LIBS -lFoundation"; AC_DEFINE(LIB_FOUNDATION_LIBRARY) AC_MSG_ERROR([
    You don't have to compile this library if you are using libFoundation,
because it already has the extensions this library provide.])
    ;;
    *)			AC_MSG_ERROR(Unknown $FOUNDATION_LIBRARY library!);;
esac
AC_MSG_RESULT(${ac_cv_foundation_library})
AC_DETERMINE_RUNTIME
])dnl
dnl
dnl
AC_DEFUN(AC_DETERMINE_RUNTIME,
[AC_REQUIRE([AC_PROG_CC])dnl
AC_REQUIRE([AC_C_CROSS])dnl
AC_MSG_CHECKING(for the Objective-C runtime)
AC_CACHE_VAL(ac_cv_objc_runtime,
[if test "$OBJC_RUNTIME" = ""; then
AC_LANG_SAVE[]dnl
AC_LANG_OBJECTIVE_C[]
AC_TRY_LINK([#include <objc/Object.h>],
[extern id objc_lookUpClass(char*);
id class = objc_lookUpClass("Object");
id obj = [class alloc];
puts([obj name]);
], ac_cv_objc_runtime=NeXT, ac_cv_objc_runtime=unknown)
# Sun has modified the NeXT's runtime, so this check determines if we are on
# Sun's runtime
if test $ac_cv_objc_runtime = NeXT; then
  AC_TRY_LINK([#include <Foundation/NSString.h>],
  [extern BOOL objc_addFilterForClass(const char *className);
  objc_addFilterForClass("NSObject");
  ], ac_cv_objc_runtime=Sun, ac_cv_objc_runtime=NeXT)dnl
fi
if test $ac_cv_objc_runtime = unknown; then
  # Save the LIBS variable
  HOLD_LIBS=$LIBS
  LIBS="$LIBS -lobjc"
  AC_TRY_LINK([#include <objc/Object.h>],
  [extern id objc_lookup_class(char*);
  id class = objc_lookup_class("Object");
  id obj = [class alloc];
  puts([obj name]);
  ], ac_cv_objc_runtime=GNU, ac_cv_objc_runtime=unknown)
  # Restore the LIBS variable
  LIBS=$HOLD_LIBS
fi
AC_LANG_RESTORE[]
else
    ac_cv_objc_runtime=$OBJC_RUNTIME
fi
])dnl
if test "`echo ${ac_cv_objc_runtime} | tr a-z A-Z`" = "GNU"; then
    OBJC_RUNTIME=GNU
    OBJC_RUNTIME_FLAG=-fgnu-runtime
    ac_cv_objc_runtime=GNU
    LIBS="$LIBS -lobjc"
    AC_DEFINE(GNU_RUNTIME)
elif test "`echo ${ac_cv_objc_runtime} | tr a-z A-Z`" = "NEXT"; then
    OBJC_RUNTIME=NeXT
    OBJC_RUNTIME_FLAG=-fnext-runtime
    ac_cv_objc_runtime=NeXT
    AC_DEFINE(NeXT_RUNTIME)
else
    echo
    rm -f conftest* confdefs* core core.* *.core
    AC_MSG_ERROR([Cannot determine the Objective-C runtime! Probably you
have to specify some additional libraries needed to link an ObjC program,
so please take a look in the config,log file to see what was the reason
the test program has not been built and try again.])
fi
AC_MSG_RESULT(${ac_cv_objc_runtime})
])dnl



