/* 
   MissingMethods.h

   Copyright (C) 1995, 1996, 1997 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#ifndef __MissingMethods_h__
#define __MissingMethods_h__

#include <Foundation/NSArray.h>
#include <Foundation/NSDictionary.h>
#include <extensions/NSConcreteNumber.h>
#include <extensions/support.h>

#if (__GNUC__ == 2) && (__GNUC_MINOR__ <= 6) && !defined(__attribute__)
#  define __attribute__(x)
#endif

#if NeXT_foundation_LIBRARY
#  include <extensions/HiddenClasses.h>
#endif

#if NeXT_foundation_LIBRARY

static inline NSRange
NSMakeRange(unsigned int location, unsigned int length)
    __attribute__((unused));

static inline NSRange
NSMakeRange(unsigned int location, unsigned int length)
{
    NSRange range;
    range.location = location;
    range.length   = length;
    return range;
}

#endif /* NeXT_foundation_LIBRARY */

#if NeXT_foundation_LIBRARY

@interface NSArray (MissingMethods)

- (NSArray*)arrayByAddingObject:(id)anObject;
- (NSArray *)arrayByAddingObjectsFromArray:(NSArray *)anotherArray;
- (NSString *)componentsJoinedByString:(NSString *)separator;

@end /* NSArray (MissingMethods) */

@interface NSMutableArray (MissingMethods)

- (void)setArray:(NSArray *)otherArray;

@end /* NSMutableArray (MissingMethods) */

#endif /* NeXT_foundation_LIBRARY */

#if GNUSTEP_BASE_LIBRARY

@interface NSArray (MissingMethods)

- initWithArray:(NSArray *)anArray copyItems:(BOOL)flag;

@end /* NSArray (MissingMethods) */

#endif /* GNUSTEP_BASE_LIBRARY */

@interface NSDictionary (MissingMethods)

#if !(GNUSTEP_BASE_LIBRARY)
+ (id)dictionaryWithObjectsAndKeys:(id)firstObject, ...;
#endif

#if NeXT_foundation_LIBRARY
- (id)initWithObjects:(NSArray*)objects forKeys:(NSArray*)keys;
#endif

@end /* NSDictionary (MissingMethods) */


@interface NSMutableDictionary (MissingMethods)
- (void)setDictionary:(NSDictionary*)otherDictionary;
@end /* NSMutableDictionary (MissingMethods) */


#if NeXT_foundation_LIBRARY || Sun_Foundation_LIBRARY

@interface NSObject (MissingMethods)
- subclassResponsibility:(SEL)aSel;
- notImplemented:(SEL)aSel;
@end /* NSObject (MissingMethods) */

#endif /* NeXT_foundation_LIBRARY */


#if NeXT_foundation_LIBRARY || GNUSTEP_BASE_LIBRARY

@interface NSString (MissingMethods)
- (NSString *)substringWithRange:(NSRange)range;
@end

#endif /* NeXT_foundation_LIBRARY */

#if NeXT_foundation_LIBRARY || GNUSTEP_BASE_LIBRARY

@interface NSNumber(MissingMethods)
- initValue:(const void*)value withObjCType:(const char*)type;
- (int)generality;
@end

#endif /* NeXT_foundation_LIBRARY */


#endif /* __MissingMethods_h__ */
